'##############################################################################
'#
'# A Mobile One Time Passwords implementation in Visual Basic .Net.
'# Learn more about mOTP at: http://motp.sourceforge.net/
'#
'# Written by Lester Hightower <lester dot hightower at Gmail>, Nov 2011
'# License: The GNU General Public License, version 3.0 (GPLv3).
'#
'# This program was written because I was able to find a reasonable MOTP
'# client for just about every conceivable platform except Windows desktop!
'# This program was written exclusively with Mono on an Ubuntu Linux desktop,
'# but that seemed to go quite well. It was tested on Linux and Windows 7.
'#
'# This program relies on IniFileVb.vb from:
'#            http://www.codeproject.com/KB/files/VB_NET_INIFile_Object.aspx
'#
'##############################################################################

Imports System.Windows.Forms
Imports System.Drawing
Imports System.Security.Cryptography
Imports System.Collections
Imports System.Text
Imports System.IO

Public Class WinVBApp
    Inherits Form
    Private AppName As String = "mOTP4dotNet"
    Private AppVersion As String = "1.0"
    Private tlpMain As TableLayoutPanel
    Private tlpConf As TableLayoutPanel
    Private tlpAbout As TableLayoutPanel
    Private tbPIN As TextBox
    Private cbProfiles As ComboBox
    Private tbOTPOutput As TextBox
    Private lbProfilesConf As ListBox
    Private tbProfNameConf As TextBox
    Private lblSecret As Label
    Private tbSecretConf As TextBox

    Public Sub New
       Me.Text = "mOTP for .Net"
       Me.Size = New Size(250, 315)
       Me.MinimumSize = Me.Size
       Me.InitUI
       Me.CenterToScreen
    End Sub

    Private Sub InitUI
        Me.tlpMain  = GetMainTLP()
        Me.tlpConf  = GetConfTLP()
        Me.tlpAbout = GetAboutTLP()
        RefreshProfilesList()
        ShowTLP(Me.tlpMain)

        Dim mainMenu As New MainMenu
        Dim mFile As MenuItem=mainMenu.MenuItems.Add("File")
        Dim mFileQuit As MenuItem=mFile.MenuItems.Add("Configuration", New EventHandler(AddressOf mnuConf_Click))
        mFile.MenuItems.Add("-")
        Dim mFileQuit As MenuItem=mFile.MenuItems.Add("Quit", New EventHandler(AddressOf mnuQuit_Click))
        Dim mHelp As MenuItem=mainMenu.MenuItems.Add("Help")
        Dim mHelpAbout As MenuItem=mHelp.MenuItems.Add("About", New EventHandler(AddressOf mnuAbout_Click))
        Me.Menu=mainMenu
    End Sub

    Private Function GetMainTLP() As TableLayoutPanel
        ' The LabelFont is the font used for our labels
        Dim LabelFont As System.Drawing.Font = new System.Drawing.Font("Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)

        ' This form's layout object is a TableLayoutPanel named tlpMain
        tlpMain = New TableLayoutPanel
        tlpMain.Name = "tlpMain"
        tlpMain.AutoSize = true
        tlpMain.BackColor = System.Drawing.Color.Transparent
        tlpMain.ColumnCount = 2
        tlpMain.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Absolute, 60F))
        tlpMain.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 100F))
        tlpMain.Dock = System.Windows.Forms.DockStyle.Fill
        tlpMain.Location = new System.Drawing.Point(0, 0)
        tlpMain.Padding = new System.Windows.Forms.Padding(2)
        For i As Integer = 1 To 5
          tlpMain.RowStyles.Add(new System.Windows.Forms.RowStyle())
        Next
        tlpMain.RowStyles.Add(new System.Windows.Forms.RowStyle(System.Windows.Forms.SizeType.Percent, 100F))
        tlpMain.TabIndex = 0
        tlpMain.Visible = false
        tlpMain.Parent = Me

        ' The tblRow integer is incremented as we build up tlpMain's rows
        Dim tblRow as Integer = 0

        ' The "title" at the top of the form -- "Get Password"
        Dim lblTitle As New Label
        lblTitle.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblTitle.Text = "Get Password"
        lblTitle.TextAlign = ContentAlignment.MiddleCenter
        lblTitle.Font = new System.Drawing.Font("Microsoft Sans Serif", 14F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)
        lblTitle.Parent = tlpMain
        tlpMain.SetColumnSpan(lblTitle,2)
        tlpMain.Controls.Add(lblTitle, 0, tblRow)
        tblRow = tblRow + 1

        ' The label for the profile selection combobox
        Dim lblSelProf As New Label
        lblSelProf.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblSelProf.Text = "Select Profile:"
        lblSelProf.TextAlign = ContentAlignment.BottomLeft
        lblSelProf.Font = LabelFont
        lblSelProf.Parent = tlpMain
        tlpMain.SetColumnSpan(lblSelProf,2)
        tlpMain.Controls.Add(lblSelProf, 0, tblRow)
        tblRow = tblRow + 1

        ' The profile selection combobox
        Me.cbProfiles = New ComboBox
        Me.cbProfiles.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        Me.cbProfiles.Parent = tlpMain
        Me.cbProfiles.DropDownStyle = ComboBoxStyle.DropDownList
        tlpMain.SetColumnSpan(Me.cbProfiles,2)
        tlpMain.Controls.Add(Me.cbProfiles, 0, tblRow)
        tblRow = tblRow + 1

        ' The label for the PIN input and the OK button line
        Dim lblPin_and_OK As New Label
        lblPin_and_OK.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblPin_and_OK.Text = "Enter PIN and click OK:"
        lblPin_and_OK.TextAlign = ContentAlignment.BottomLeft
        lblPin_and_OK.Font = LabelFont
        lblPin_and_OK.Parent = tlpMain
        tlpMain.SetColumnSpan(lblPin_and_OK,2)
        tlpMain.Controls.Add(lblPin_and_OK, 0, tblRow)
        tblRow = tblRow + 1

        ' The TextBox for the PIN input
        Me.tbPIN = New TextBox
        Me.tbPIN.PasswordChar = "*"
        Me.tbPIN.Text = ""
        Me.tbPIN.Parent = tlpMain
        tlpMain.Controls.Add(Me.tbPIN, 0, tblRow)

        ' The OK button line
        Dim bOK As New Button
        bOK.Text = "OK"
        bOK.Parent = tlpMain
        bOK.Anchor = AnchorStyles.Left Or AnchorStyles.Right Or AnchorStyles.Top
        tlpMain.Controls.Add(bOK, 1, tblRow)
        AddHandler bOK.Click, AddressOf Me.GetPassword
        tblRow = tblRow + 1

        ' The TextBox that will be populated with the OTP when it's calculated
        Me.tbOTPOutput = New TextBox
        Me.tbOTPOutput.Anchor = AnchorStyles.Left Or AnchorStyles.Right
	Me.tbOTPOutput.BackColor = System.Drawing.SystemColors.Control
        Me.tbOTPOutput.BorderStyle = System.Windows.Forms.BorderStyle.None
        Me.tbOTPOutput.ReadOnly = true
        Me.tbOTPOutput.Text = "OTP will appear here..."
        Me.tbOTPOutput.TextAlign = HorizontalAlignment.Center
        Me.tbOTPOutput.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)
        Me.tbOTPOutput.Parent = tlpMain
        tlpMain.SetColumnSpan(Me.tbOTPOutput,2)
        tlpMain.Controls.Add(Me.tbOTPOutput, 0, tblRow)
        tblRow = tblRow + 1

        return tlpMain
    End Function

    Private Function GetConfTLP() As TableLayoutPanel
        ' The LabelFont is the font used for our labels
        Dim LabelFont As System.Drawing.Font = new System.Drawing.Font("Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)

        ' This form's layout object is a TableLayoutPanel named tlpConf
        tlpConf = New TableLayoutPanel
        tlpConf.Name = "tlpConf"
        tlpConf.AutoSize = true
        tlpConf.BackColor = System.Drawing.Color.Transparent
        tlpConf.ColumnCount = 2
        tlpConf.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 50F))
        tlpConf.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 50F))
        tlpConf.Dock = System.Windows.Forms.DockStyle.Fill
        tlpConf.Location = new System.Drawing.Point(0, 0)
        tlpConf.Padding = new System.Windows.Forms.Padding(0)
        tlpConf.Margin = new System.Windows.Forms.Padding(0)
        For i As Integer = 1 To 8
          tlpConf.RowStyles.Add(new System.Windows.Forms.RowStyle())
        Next
        tlpConf.RowStyles.Add(new System.Windows.Forms.RowStyle(System.Windows.Forms.SizeType.Percent, 100F))
        tlpConf.TabIndex = 0
        tlpConf.Visible = false
        tlpConf.Parent = Me

        ' The tblRow integer is incremented as we build up tlpConf's rows
        Dim tblRow as Integer = 0

        ' The "title" at the top of the form -- "Get Password"
        Dim lblTitle As New Label
        lblTitle.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblTitle.Text = "Configuration"
        lblTitle.TextAlign = ContentAlignment.MiddleCenter
        lblTitle.Font = new System.Drawing.Font("Microsoft Sans Serif", 14F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)
        lblTitle.Parent = tlpConf
        tlpConf.SetColumnSpan(lblTitle,2)
        tlpConf.Controls.Add(lblTitle, 0, tblRow)
        tblRow = tblRow + 1

        ' The label for the profile selection combobox
        Dim lblSelProf As New Label
        lblSelProf.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblSelProf.Text = "Profiles:"
        lblSelProf.TextAlign = ContentAlignment.BottomLeft
        lblSelProf.Font = LabelFont
        lblSelProf.Parent = tlpConf
        tlpConf.SetColumnSpan(lblSelProf,2)
        tlpConf.Controls.Add(lblSelProf, 0, tblRow)
        tblRow = tblRow + 1

        ' The profile selection ListBox
        Me.lbProfilesConf = New ListBox
        Me.lbProfilesConf.MultiColumn = False
        Me.lbProfilesConf.SelectionMode = SelectionMode.One
        Me.lbProfilesConf.Anchor = AnchorStyles.Left Or AnchorStyles.Right Or AnchorStyles.Bottom Or AnchorStyles.Top
        'Me.lbProfilesConf.IntegralHeight = false
        Me.lbProfilesConf.Height = Int(Me.lbProfilesConf.ItemHeight * 3.5)
        Me.lbProfilesConf.Parent = tlpConf
        tlpConf.SetColumnSpan(Me.lbProfilesConf,2)
        tlpConf.Controls.Add(Me.lbProfilesConf, 0, tblRow)
        AddHandler Me.lbProfilesConf.SelectedValueChanged, AddressOf lbProfilesConf_SelectedValueChanged
        tblRow = tblRow + 1

        ' The profile name label
        Dim lblProfName As New Label
        lblProfName.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblProfName.Text = "Profile Name:"
        lblProfName.TextAlign = ContentAlignment.BottomLeft
        lblProfName.Font = LabelFont
        lblProfName.Parent = tlpConf
        tlpConf.SetColumnSpan(lblProfName,2)
        tlpConf.Controls.Add(lblProfName, 0, tblRow)
        tblRow = tblRow + 1

        ' The profile name text box
        Me.tbProfNameConf = New TextBox
        Me.tbProfNameConf.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        Me.tbProfNameConf.Parent = tlpConf
        tlpConf.SetColumnSpan(Me.tbProfNameConf,2)
        tlpConf.Controls.Add(Me.tbProfNameConf, 0, tblRow)
        tblRow = tblRow + 1

        ' The secret label
        Me.lblSecret = New Label
        lblSecret.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblSecret.Text = "Secret:"
        lblSecret.TextAlign = ContentAlignment.BottomLeft
        lblSecret.Font = LabelFont
        lblSecret.Parent = tlpConf
        tlpConf.SetColumnSpan(lblSecret,2)
        tlpConf.Controls.Add(lblSecret, 0, tblRow)
        tblRow = tblRow + 1

        ' The secret text box
        Me.tbSecretConf = New TextBox
        Me.tbSecretConf.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        Me.tbSecretConf.Parent = tlpConf
        tlpConf.SetColumnSpan(Me.tbSecretConf,2)
        tlpConf.Controls.Add(Me.tbSecretConf, 0, tblRow)
        AddHandler Me.tbSecretConf.TextChanged, AddressOf Me.tbSecretConf_TextChanged
        tblRow = tblRow + 1

        ' The Delete button
        Dim bConfDel As New Button
        bConfDel.Text = "Delete"
        bConfDel.Parent = tlpConf
        bConfDel.Anchor = AnchorStyles.Left Or AnchorStyles.Right Or AnchorStyles.Top
        tlpConf.Controls.Add(bConfDel, 0, tblRow)
        AddHandler bConfDel.Click, AddressOf Me.bConfDel_Click
        ' The Save button
        Dim bConfSave As New Button
        bConfSave.Text = "Save"
        bConfSave.Parent = tlpConf
        bConfSave.Anchor = AnchorStyles.Left Or AnchorStyles.Right Or AnchorStyles.Top
        tlpConf.Controls.Add(bConfSave, 1, tblRow)
        AddHandler bConfSave.Click, AddressOf Me.bConfSave_Click
        tblRow = tblRow + 1

        ' The Exit Configuration
        Dim bExitConf As New Button
        bExitConf.Text = "Exit Configuration Screen"
        bExitConf.Anchor = AnchorStyles.Left Or AnchorStyles.Right Or AnchorStyles.Top Or AnchorStyles.Bottom
        bExitConf.Parent = tlpConf
        tlpConf.SetColumnSpan(bExitConf,2)
        tlpConf.Controls.Add(bExitConf, 0, tblRow)
        AddHandler bExitConf.Click, AddressOf Me.bExitConf_Click
        tblRow = tblRow + 1

        return tlpConf
    End Function

    Private Function GetAboutTLP() As TableLayoutPanel
        ' This form's layout object is a TableLayoutPanel named tlpAbout
        tlpAbout = New TableLayoutPanel
        tlpAbout.Name = "tlpAbout"
        tlpAbout.AutoSize = true
        tlpAbout.BackColor = System.Drawing.Color.Transparent
        tlpAbout.ColumnCount = 1
        tlpAbout.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 100F))
        tlpAbout.Dock = System.Windows.Forms.DockStyle.Fill
        tlpAbout.Location = new System.Drawing.Point(0, 0)
        tlpAbout.Padding = new System.Windows.Forms.Padding(2)
        tlpAbout.RowCount = 3
        tlpAbout.RowStyles.Add(new System.Windows.Forms.RowStyle())
        Dim rowstyle As System.Windows.Forms.RowStyle = new System.Windows.Forms.RowStyle(SizeType.Percent)
        rowstyle.Height=90
        tlpAbout.RowStyles.Add(rowstyle)
        tlpAbout.RowStyles.Add(new System.Windows.Forms.RowStyle())
        tlpAbout.TabIndex = 0
        tlpAbout.Visible = false
        tlpAbout.Parent = Me

        Dim tblRow As Integer = 0

        ' The "title" at the top of the About form
        Dim lblAboutTitle As New Label
        lblAboutTitle.AutoSize = false
        lblAboutTitle.Anchor = AnchorStyles.Left Or AnchorStyles.Right
        lblAboutTitle.Text = Me.AppName + " v" + Me.AppVersion
        lblAboutTitle.TextAlign = ContentAlignment.MiddleCenter
        lblAboutTitle.Font = new System.Drawing.Font("Microsoft Sans Serif", 14F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)
        lblAboutTitle.Parent = tlpMain
        tlpAbout.Controls.Add(lblAboutTitle, 0, tblRow)
        tblRow = tblRow + 1

        ' The About message
        Dim lblAbout As New Label
        lblAbout.AutoSize = True
        lblAbout.Dock = System.Windows.Forms.DockStyle.Fill
        lblAbout.Text = "A Mobile One Time Password Generator for .Net" + _
		vbNewLine + vbNewLine + _
		"Written by Lester Hightower" + _
		vbNewLine + _
		"Licensed under the GPLv3" + _
		vbNewLine + vbNewLine + _
		"Learn more about MOTP at: http://motp.sourceforge.net/"
        lblAbout.TextAlign = ContentAlignment.MiddleCenter
        lblAbout.Font = new System.Drawing.Font("Microsoft Sans Serif", 11F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)
        lblAbout.Parent = tlpMain
        tlpAbout.Controls.Add(lblAbout, 0, tblRow)
        tblRow = tblRow + 1

        ' The OK button line
        Dim bOK As New Button
        bOK.Text = "OK"
        bOK.Parent = tlpAbout
        bOK.Anchor = AnchorStyles.Right Or AnchorStyles.Bottom
        tlpAbout.Controls.Add(bOK, 0, tblRow)
        AddHandler bOK.Click, AddressOf Me.CloseAbout_Click
        tblRow = tblRow + 1

        return tlpAbout
    End Function

    Private Sub ShowTLP(tlp As TableLayoutPanel)
      tlpMain.Visible = false
      tlpConf.Visible = false
      tlpAbout.Visible = false
      tlp.Visible = true
    End Sub

    Private Sub CloseAbout_Click(ByVal sender As Object,ByVal e As EventArgs)
      ShowTLP(Me.tlpMain)
    End Sub

    Private Sub bExitConf_Click(ByVal sender As Object,ByVal e As EventArgs)
      ShowTLP(Me.tlpMain)
    End Sub

    Private Sub bConfDel_Click(ByVal sender As Object,ByVal e As EventArgs)
      Dim ChosenProfile As String
      Try ' This try catches an empty *.ini or anything else odd
        ChosenProfile = Me.lbProfilesConf.SelectedItems(0).ToString()
      Catch
        Exit Sub
      End Try
      Dim profiles As IniFile = GetProfiles()
      Dim style As MsgBoxStyle = MsgBoxStyle.YesNo Or _
				MsgBoxStyle.ApplicationModal
      Dim r As MsgBoxResult = MsgBox("Are you sure that you wish to delete" + _
		vbNewLine + _
		"the profile named """ + ChosenProfile + """?" + _
		vbNewLine + vbNewLine + _
		"This action cannot be undone!", _
		style, "Delete a Profile?")
      If r = MsgBoxResult.Yes Then
        profiles.RemoveSection(ChosenProfile)
        profiles.Save(GetIniFilename())
        RefreshProfilesList()
      end if
    End Sub

    Private Sub bConfSave_Click(ByVal sender As Object,ByVal e As EventArgs)
      If (Me.tbProfNameConf.Text <> "") Then
        Dim profname As String = Me.tbProfNameConf.Text
        Dim profiles As IniFile = GetProfiles()
        If (profiles Is Nothing) Then ' If the file doesn't exist
          profiles = New IniFile
        End If
        Dim secret As String =  Me.tbSecretConf.Text
        profiles.SetKeyValue(profname,"secret",secret)
        profiles.Save(GetIniFilename())
        RefreshProfilesList()
      End If
    End Sub

    Private Sub tbSecretConf_TextChanged(ByVal sender As Object,ByVal e As EventArgs)
      Me.lblSecret.Text = "Secret (" + _
			 tbSecretConf.TextLength.ToString() + " chars):"
    End Sub

    Private Sub lbProfilesConf_SelectedValueChanged(ByVal sender As Object,ByVal e As EventArgs)
      If Me.lbProfilesConf.SelectedIndex <> -1 Then
        ' ListBoxes are multicolumn and so we need the data in column 0
        Dim profname As String = Me.lbProfilesConf.SelectedItems(0).ToString()
        Me.tbProfNameConf.Text = profname
        Dim profiles as IniFile = GetProfiles()
        Dim Secret As String = profiles.GetKeyValue(profname,"secret")
        Me.tbSecretConf.Text = Secret
      End If
    End Sub

    Private Sub mnuQuit_Click(ByVal sender As Object, ByVal e As EventArgs)
       Me.Close
    End Sub

    Private Sub mnuAbout_Click(ByVal sender As Object, ByVal e As EventArgs)
      ShowTLP(Me.tlpAbout)
    End Sub

    Private Sub mnuConf_Click(ByVal sender As Object, ByVal e As EventArgs)
      ShowTLP(Me.tlpConf)
    End Sub

    Private Sub GetPassword(ByVal sender As Object, ByVal e As EventArgs)
       Dim ChosenProfile As String = Me.cbProfiles.Text
       Dim PIN As String = Me.tbPIN.Text
       Dim profiles as IniFile = GetProfiles()
       If (profiles Is Nothing) Then
         Me.tbOTPOutput.Text = "No Profiles Configured!"
         Exit Sub
       End If
       Dim Secret As String = profiles.GetKeyValue(ChosenProfile,"secret")
       Dim mOTP_Now As Integer = UnixTime(DateTime.Now) / 10
       Dim authkey As String = Convert.ToString(mOTP_Now) + Secret + PIN
       Dim md5hash As String = CalculateMD5Hash(authkey)
       Dim otp As String = md5hash.substring(0,6)
       Me.tbOTPOutput.Text = otp
       Me.tbPIN.Text = "" ' Empty the PIN box for security reasons

       if (0) then
         Console.WriteLine("Profile=" + ChosenProfile)
         Console.WriteLine("PIN=" + PIN)
         Console.WriteLine("Secret=" + Secret)
         Console.WriteLine("mOTP_Now=" + Convert.ToString(mOTP_Now))
         Console.WriteLine("md5hash=" + md5hash)
         Console.WriteLine("otp=" + otp)
       end if
    End Sub

    Private Function CalculateMD5Hash(ByVal str As String) As String
      Dim md5 As MD5 = MD5.Create()
      Dim hash As Byte() = md5.ComputeHash(Encoding.UTF8.GetBytes(str))
      Dim sb As StringBuilder = new StringBuilder()
      for i As Integer = 0 To hash.Length - 1
          sb.Append(hash(i).ToString("x2"))
      next
      return sb.ToString()
    End Function

    Private Function UnixTime(dDate As DateTime) As Integer
      Dim epoch As DateTime = new DateTime(1970,1,1,0,0,0,DateTimeKind.Utc)
      Dim myUTC As DateTime = dDate.ToUniversalTime()
      Dim RetVal As DateTime= new DateTime(myUTC.Ticks - epoch.Ticks)
      return RetVal.Ticks / TimeSpan.TicksPerSecond
    End Function

    Private Sub RefreshProfilesList
        Dim profiles() As String = GetProfileNames()
        Me.cbProfiles.Items.Clear()
        Me.cbProfiles.Items.AddRange(profiles)
        Me.lbProfilesConf.Items.Clear()
        Me.lbProfilesConf.Items.AddRange(profiles)
        if (Me.cbProfiles.Items.Count > 0) then
          Me.cbProfiles.SelectedIndex = 0
          Me.lbProfilesConf.SelectedIndex = 0
          Me.lbProfilesConf.TopIndex = 0
        end if
    End Sub

    Private Function GetProfileNames As String()
       Dim profile_names As New ArrayList
       Dim profiles as IniFile = GetProfiles()
       If (Not profiles Is Nothing) Then
         Dim sectionData as IniFile.IniSection
         For Each sectionData in profiles.Sections
           profile_names.Add(sectionData.Name)
         Next
       End If
       profile_names.Sort()
       return DirectCast(profile_names.ToArray(GetType(String)), String())
    End Function

    Private Function GetIniFilename As String
       Dim exepath As String = System.Reflection.Assembly.GetExecutingAssembly.Location()
       Dim exedir As String = System.IO.Path.GetDirectoryName(exepath)
       Dim IniFile as String = Nothing
       if (System.IO.Path.GetExtension(exepath).ToLower = "exe") then
         IniFile = System.IO.Path.ChangeExtension(exepath,"ini")
       else
         IniFile = exedir + System.IO.Path.DirectorySeparatorChar + _
		System.IO.Path.GetFileNameWithoutExtension(exepath) + ".ini"
       end if
       'Console.WriteLine("IniFile=" + IniFile)
       return IniFile
    End Function

    Private Function GetProfiles As IniFile
       Dim IniFile As String = GetIniFilename()
       If (Not File.Exists(IniFile))
         return Nothing
       end if
       Try
         Dim ini As New IniFile
         ini.Load(IniFile)
         return ini
       Catch
         return Nothing
       End Try
    End Function

    Public Shared Sub Main
        Application.Run(New WinVBApp)
    End Sub

End Class


'###########################################################################
'###########################################################################
'###########################################################################


